@@ -104,7 +104,7 @@ def _build_parser():
         "--skip_bids_validation",
         "--skip-bids-validation",
         action="store_true",
-        default=True,
+        default=False,
         help="assume the input dataset is BIDS compliant and skip the validation",
     )
     g_bids.add_argument(
@@ -122,11 +122,11 @@ def _build_parser():
     # Re-enable when option is actually implemented
     # g_bids.add_argument('-r', '--run-id', action='store', default='single_run',
     #                     help='select a specific run to be processed')
+    #g_bids.add_argument(
+       # "-t", "--task-id", action="store", help="select a specific task to be processed"
+   #)
     g_bids.add_argument(
-        "-t", "--task-id", action="store", help="select a specific task to be processed"
-    )
-    g_bids.add_argument(
-        "--echo-idx",
+       "--echo-idx",
         action="store",
         type=int,
         help="select a specific echo to be processed in a multiecho series",
@@ -286,7 +286,7 @@ any spatial references."""
         required=False,
         action="store",
         default=1,
-        type=int,
+        type=float,
         help="relative scale between asl and M0.",
     )
     g_conf.add_argument(
@@ -308,7 +308,7 @@ any spatial references."""
         "--smooth_kernel",
         action="store",
         default=5,
-        type=int,
+        type=float,
         help="Smoothing kernel for the M0 image(s)",
     )
 
