@@ -27,8 +27,9 @@ DEFAULT_MEMORY_MIN_GB = config.DEFAULT_MEMORY_MIN_GB
 LOGGER = config.loggers.workflow
 
 
-def init_asl_geref_wf(omp_nthreads,mem_gb,metadata,bids_dir,brainmask_thresh=0.5,pre_mask=False, name="asl_gereference_wf",
-    gen_report=False):
+def init_asl_geref_wf(omp_nthreads,mem_gb,metadata,bids_dir,
+               smooth_kernel=5,brainmask_thresh=0.5,pre_mask=False, 
+               name="asl_gereference_wf",gen_report=False):
 
     workflow = Workflow(name=name)
     workflow.__desc__ = """\
@@ -57,7 +58,7 @@ First, a reference volume and its skull-stripped version were generated.
         name="outputnode",
     )
 
-    gen_ref = pe.Node(GeReferenceFile(bids_dir=bids_dir, in_metadata=metadata),
+    gen_ref = pe.Node(GeReferenceFile(bids_dir=bids_dir,fwhm=smooth_kernel,in_metadata=metadata),
                omp_nthreads=1,mem_gb=1,name='gen_ge_ref')
     gen_ref.base_dir=os.getcwd()
     skull_strip_wf =  pe.Node(
@@ -525,6 +526,8 @@ class _GenerateReferenceInputSpec(BaseInterfaceInputSpec):
     input_image = File(
         exists=True, mandatory=True, desc="input images"
     )
+    fwhm =traits.Float(exists=False, mandatory=False,default_value=0,
+                              desc='smoothing kernel for m0')
     
 
 class _GenerateReferenceOutputSpec(TraitedSpec):
@@ -543,18 +546,19 @@ class GenerateReference(SimpleInterface):
 
     def _run_interface(self, runtime):
         self._results["out_file"] = gen_reference(
-            in_img=self.inputs.input_image)
+            in_img=self.inputs.input_image,fwhm=self.inputs.fwhm)
         return runtime
 
 
 
 
-def gen_reference(in_img, newpath=None):
+def gen_reference(in_img,fwhm=5,newpath=None):
 
     """generate reference for a GE scan with few volumes."""
     import nibabel as nb 
     import numpy as np
     import os 
+    from nibabel.processing import smooth_image
     newpath = Path(newpath or ".")
     ss=check_img(in_img)
     if ss == 0: 
@@ -562,9 +566,10 @@ def gen_reference(in_img, newpath=None):
     else: 
         nii = nb.load(in_img).get_fdata()
         ref_data=np.mean(nii,axis=3)
-    
     new_file = nb.Nifti1Image(dataobj=ref_data,header=nb.load(in_img).header,
              affine=nb.load(in_img).affine)
+
+    new_file = smooth_image(new_file,fwhm=fwhm)
     out_file = fname_presuffix('aslref', suffix="_reference.nii.gz", newpath=str(newpath.absolute()))
     new_file.to_filename(out_file)
     return out_file
@@ -631,6 +636,7 @@ class _GeReferenceFileInputSpec(BaseInterfaceInputSpec):
     bids_dir=traits.Str(exits=True,mandatory=True,desc=' bids directory')
     ref_file = File(exists=False,mandatory=False, desc="ref file")
     m0_file = File(exists=False,mandatory=False, desc="m0 file")
+    fwhm  = traits.Float(exits=False,mandatory=False,default_value=5,desc="smoothing kernel for M0")
     
 
 class _GeReferenceFileOutputSpec(TraitedSpec):
@@ -656,44 +662,82 @@ class GeReferenceFile(SimpleInterface):
         idasl = aslcontext['volume_type'].tolist()
         m0list = [i for i in range(0, len(idasl)) if idasl[i] == 'm0scan']
         deltamlist = [i for i in range(0, len(idasl)) if idasl[i] == 'deltam']
+        controllist = [i for i in range(0, len(idasl)) if idasl[i] == 'control']
+        labellist = [i for i in range(0, len(idasl)) if idasl[i] == 'label']
         cbflist = [i for i in range(0, len(idasl)) if idasl[i] == 'CBF']
+
         allasl = nb.load(self.inputs.in_file)
         dataasl = allasl.get_fdata()
-
-        if self.inputs.in_metadata['M0'] != "True" and type(self.inputs.in_metadata['M0']) != int :
-            m0file=os.path.abspath(self.inputs.bids_dir+'/'+self.inputs.in_metadata['M0'])
-            reffile = gen_reference(m0file,newpath=runtime.cwd)
+         
+        if self.inputs.in_metadata['M0Type'] == 'Separate':
+            m0file = self.inputs.in_file.replace("asl.nii.gz","m0scan.nii.gz")
+            m0file_metadata=readjson(m0file.replace('nii.gz','json'))
+            aslfile_linkedM0 = os.path.abspath(self.inputs.bids_dir+'/'+m0file_metadata['IntendedFor'])
+            reffile = gen_reference(m0file,fwhm=self.inputs.fwhm, newpath=runtime.cwd)
             m0file = reffile
-        if self.inputs.in_metadata['M0'] == "True":
+        
+        elif self.inputs.in_metadata['M0Type'] == "Included":
             modata2 = dataasl[:, :, :, m0list]
             m0filename=fname_presuffix(self.inputs.in_file,
                                                     suffix='_mofile', newpath=os.getcwd())
             m0obj = nb.Nifti1Image(modata2, allasl.affine, allasl.header)
             m0obj.to_filename(m0filename)
-            reffile = gen_reference(m0filename,newpath=runtime.cwd)
+            reffile = gen_reference(m0filename,fwhm=self.inputs.fwhm,newpath=runtime.cwd)
             m0file = reffile
 
-        elif type(self.inputs.in_metadata['M0']) == int or  type(self.inputs.in_metadata['M0']) == float :
-            m0num=np.float(self.inputs.in_metadata['M0'])
-            modata2 = dataasl[:, :, :, deltamlist]
-            m0filename=fname_presuffix(self.inputs.in_file,
-                                                    suffix='_mofile', newpath=os.getcwd())
-            m0obj = nb.Nifti1Image(modata2, allasl.affine, allasl.header)
+        elif self.inputs.in_metadata["M0Type"] == "Estimate":
+            m0num = np.float(self.inputs.in_metadata['M0Estimate'])
+            if len(deltamlist) > 0:
+                modata2 = dataasl[:, :, :, deltamlist]
+            elif len(controllist) > 0 :
+                modata2 = dataasl[:, :, :, controllist]
+            elif len(cbflist) > 0 :
+                modata2 = dataasl[:, :, :, cbflist]
+
+            m0filename = fname_presuffix(self.inputs.in_file,
+                                                    suffix='_m0file', newpath=os.getcwd())
+            if len(modata2.shape) > 3:
+                mdata = np.mean(modata2,axis=3)
+            else: 
+                mdata = modata2
+
+            m0file_data=m0num * np.ones_like(mdata)
+
+            m0obj = nb.Nifti1Image(m0file_data, allasl.affine, allasl.header)
             m0obj.to_filename(m0filename)
-            reffile = gen_reference(m0filename,newpath=runtime.cwd)
-            m0file_data=m0num * np.ones_like(nb.load(reffile).get_fdata())
+            m0file = m0filename
 
-            m0filename1=fname_presuffix(self.inputs.in_file,
-                                                    suffix='_mofile', newpath=os.getcwd())
-            m0obj1 = nb.Nifti1Image(m0file_data, allasl.affine, allasl.header)
-            m0obj1.to_filename(m0filename1)
-            m0file = gen_reference(m0filename1,newpath=runtime.cwd)
-            reffile = m0file
+            reffilename = fname_presuffix(self.inputs.in_file,
+                                                    suffix='_refile', newpath=os.getcwd())
+            refobj = nb.Nifti1Image(mdata, allasl.affine, allasl.header)
 
-        elif len(cbflist) > 0 :
-            reffile=gen_reference(self.inputs.in_file,newpath=runtime.cwd)
-            m0file = reffile 
-        
+            refobj.to_filename(reffilename)
+            reffile = gen_reference(reffilename,fwhm=0,newpath=runtime.cwd)
+
+        elif self.inputs.in_metadata["M0Type"] == "Absent":
+            if len(controllist) > 0 :
+                modata2 = dataasl[:, :, :, controllist]
+            m0filename = fname_presuffix(self.inputs.in_file,
+                                                    suffix='_m0file', newpath=os.getcwd())
+            if len(modata2.shape) > 3:
+                mdata = np.mean(modata2,axis=3)
+            else: 
+                mdata = modata2
+
+            m0obj = nb.Nifti1Image(mdata, allasl.affine, allasl.header)
+            m0obj.to_filename(m0filename)
+            m0file = m0filename
+            m0file  = gen_reference(reffilename,fwhm=self.inputs.fwhm,newpath=runtime.cwd)
+
+            reffilename = fname_presuffix(self.inputs.in_file,
+                                                    suffix='_refile', newpath=os.getcwd())
+            refobj = nb.Nifti1Image(mdata, allasl.affine, allasl.header)
+
+            refobj.to_filename(reffilename)
+            reffile = gen_reference(reffilename,fwhm=0,newpath=runtime.cwd)
+
+        else:
+            raise RuntimeError("no path way to obtain real m0scan")
         self._results['ref_file']=reffile
         self._results['m0_file']=m0file
         self.inputs.ref_file = os.path.abspath(self._results['ref_file'])
